//----------------------------------------------------------------------------------------------

// CopyJob.h

//----------------------------------------------------------------------------------------------

#pragma once


typedef enum {kForFlatFile,kHandleLayers} OpenClipCopyMode;

class CopyJob {
public:
  CopyJob(DWindow *TheWndw);  // default mode is kForFlatFile
  
  void SetCopyMode(OpenClipCopyMode TheMode);
  void SetHandleGroups(bool SetIt);     // default is false
  void SetClearPasteboard(bool SetIt);  // default is false
  
  // a do-nothing place-holder, override this to output Z height for 2D objects
  // note that this will fire off for arrows as well
  virtual void HandleZHeight(const PDObject &TheObj,NSMutableDictionary *ThisObj) const;

protected:
  CDPoint CorrectPoint(CDPoint ThePt) const;
  
  void SetScale(double TheScale,NSMutableDictionary *TheDict) const;
  void SetGlobalOrigin(double X,double Y,NSMutableDictionary *TheDict) const;
  void SetInsertionPoint(double X,double Y,NSMutableDictionary *TheDict) const;
  
  void SetLayerName(CStr255 TheName,NSMutableDictionary *TheNSLayer) const;
  void SetLayerColor(RGBColor TheColor,NSMutableDictionary *TheNSLayer) const;
  void SetLayerScale(double TheScale,NSMutableDictionary *TheNSLayer) const;
  
  void SetObjectType(NSString *TheType,NSMutableDictionary *TheObj) const;
  
  void AddXYPoint(double X,double Y,NSMutableArray *ThePts) const;
  void AddXYZPoint(double X,double Y,double Z,NSMutableArray *ThePts) const;
  void AddPointsToObject(NSMutableArray *Points,NSMutableDictionary *TheObj) const;
  void SetObjectZ(double Z,NSMutableDictionary *TheObj) const;
  
  NSMutableDictionary *RGBADictionary(RGBColor TheColor,double Alpha) const;
  void SetPenColor(RGBColor PenColor,double PenAlpha,NSMutableDictionary *TheObj) const;
  void SetFillColor(const PDObject &TheObj,NSMutableDictionary *ThisObj) const;
  
  void SetLineDash(const PDObject &TheObj,NSMutableDictionary *ThisObj) const;
  void SetPenSize(float TheSize,NSMutableDictionary *TheObj) const;
  
  void SetRotationAngle(double TheAngle,NSMutableDictionary *TheObj) const;
  void SetStartAngle(double TheAngle,NSMutableDictionary *TheObj) const;
  void SetArcAngle(double TheAngle,NSMutableDictionary *TheObj) const;
  void SetCornerRadius(double TheRadius,NSMutableDictionary *TheObj) const;
  void SetClosed(bool IsClosed,NSMutableDictionary *TheObj) const;
  
  bool GetArrowObj(const CLineEndType &TheArrow,RGBColor PenColor,double Alpha,
                   float PenSize,PDObject &ThisObj) const;
  void SetArrowsBothEnds(const CLineEndType &TheArrow,RGBColor ThePenColor,double PenAlpha,
                         float ThePenSize,NSMutableDictionary *TheObj) const;
  void SetArrowAtStart(const CLineEndType &TheArrow,RGBColor ThePenColor,double PenAlpha,
                       float ThePenSize,NSMutableDictionary *TheObj) const;
  void SetArrowAtEnd(const CLineEndType &TheArrow,RGBColor ThePenColor,double PenAlpha,
                     float ThePenSize,NSMutableDictionary *TheObj) const;
  void SetArrows(const PDObject &ThisObj,RGBColor ThePenColor,double PenAlpha,
                 float ThePenSize,NSMutableDictionary *TheObj) const;
  
  virtual void CreatePointObj(const PDObject &TheObj,NSMutableDictionary *ThisObj) const;
  void CreateLineObj(const PDObject &TheObj,NSMutableDictionary *ThisObj) const;
  void CreateRectangleObj(const PDObject &TheObj,NSMutableDictionary *ThisObj) const;
  void CreateEllipseObj(const PDObject &TheObj,NSMutableDictionary *ThisObj) const;
  void CreatePolygonObj(const PDObject &TheObj,NSMutableDictionary *ThisObj) const;
  void AddBSplineAsAdvanced(const PDObject &TheObj,NSMutableDictionary *MainObj) const;
  void CreateBezierObj(const PDObject &TheObj,NSMutableDictionary *ThisObj) const;
  
  void CreateGroupObj(const PDObject &TheObj,NSMutableDictionary *ThisObj) const;
  
  virtual void AddObjectToList(const PDObject &TheObj,NSMutableArray *TheNSList) const;
  virtual void CheckList(ListRecord TheList,bool InAttachObj,NSMutableArray *TheNSList) const;
  virtual void CheckObjS(ObjSpec TheObjS,NSMutableArray *TheNSList);
  
  void WriteOutFile(NSMutableDictionary *TheDocument);
  virtual void WriteToPasteboard(NSMutableDictionary *TheDocument);
  virtual void WriteOutFlatFile(NSMutableDictionary *TheDocument);
  virtual void WriteOutLayers(NSMutableDictionary *TheDocument);

public:  
  virtual void WriteOut(void);

protected:  
  DWindow             *mTheWndw;
  double               mPgSzY;
  double               mScale;            // default 1.0
  OpenClipCopyMode     mMode;             // default kForFlatFile
  bool                 mHandleGroups;     // default false
  bool                 mClearPasteboard;  // default false
};